/*
 * Copyright (c) 2009 Dukascopy (Suisse) SA. All Rights Reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * -Redistribution of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 * 
 * -Redistribution in binary form must reproduce the above copyright notice, 
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 * 
 * Neither the name of Dukascopy (Suisse) SA or the names of contributors may 
 * be used to endorse or promote products derived from this software without 
 * specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any kind. ALL 
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING
 * ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. DUKASCOPY (SUISSE) SA ("DUKASCOPY")
 * AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE
 * AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL DUKASCOPY OR ITS LICENSORS BE LIABLE FOR ANY LOST 
 * REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY 
 * OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, 
 * EVEN IF DUKASCOPY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
package singlejartest;

import java.util.ArrayList;
import java.util.List;

import com.dukascopy.api.*;

import singlejartest.data.Price;
import singlejartest.data.Tick;

public class DataService implements IStrategy {
	private IContext context;
    private IHistory history;
    
    public void onStart(IContext context) throws JFException {
    	this.context = context;
    	this.history = context.getHistory();
    	System.out.println("Start...");
        // subscribe the instrument that we are going to work with
    	Dukascopy.LOGGER.info("Subscribing instruments...");
        context.setSubscribedInstruments(DataCenter.PairSet);
        int k = 10;
        while (!context.getSubscribedInstruments().containsAll(DataCenter.PairSet)) {
            try {
                Dukascopy.LOGGER.info("Instruments not subscribed yet " + k);
                Thread.sleep(200);
            } catch (InterruptedException e) {
            	Dukascopy.LOGGER.error(e.getMessage());
            }
            k--;
        }
    }

    public void Stop() {
    	context.stop();
    }
    
    public List<Price> getSnapshotData(String instrument, Period period, long timeFrom, long timeTo) {
    	List<Price> snapshot = new ArrayList<>();
    	
		try {
			List<IBar> bars = history.getBars(Instrument.fromString(instrument), period, OfferSide.BID, timeFrom, timeTo);
			for(IBar bar : bars) {
	        	Price p = new Price();
	        	p.Time = bar.getTime();
	        	p.Close = bar.getClose();
	        	snapshot.add(p);
	        }
		} catch (JFException e) {
			e.printStackTrace();
		}
        
        System.out.println(instrument + " has " + snapshot.size() + "bars.");
        
        return snapshot;
    }
    
    public void onStop() throws JFException {
        System.out.println("Stopped.");
    }

    public void onTick(Instrument instrument, ITick tick) throws JFException {
    	Tick t = DataCenter.TickData.get(instrument.toString());
    	if(t != null) {
    		t.Ask = tick.getAsk();
        	t.Bid = tick.getBid();
        	// t.Time = tick.getTime();
    	}
    	
    }

    public void onBar(Instrument instrument, Period period, IBar askBar, IBar bidBar) {
    }

    public void onMessage(IMessage message) throws JFException {
    }

    public void onAccount(IAccount account) throws JFException {
    }
}