package singlejartest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.dukascopy.api.system.ClientFactory;
import com.dukascopy.api.system.IClient;
import com.dukascopy.api.system.ISystemListener;

public class Dukascopy {
    public static final Logger LOGGER = LoggerFactory.getLogger(Dukascopy.class);

    private static String jnlpUrl = "http://platform.dukascopy.com/demo/jforex.jnlp";
    private static String userName = "DEMO10037qfMBcEU";
    private static String password = "qfMBc";

    public static IClient client;    
    public static DataService dataService;
    
    public static void Stop() {
    	client.disconnect();
    }
    
    public static void Start() throws Exception {
    	// preapre data
    	DataCenter.Load();
    	
        //get the instance of the IClient interface
        client = ClientFactory.getDefaultInstance();
        //set the listener that will receive system events
        client.setSystemListener(new ISystemListener() {
            private int lightReconnects = 3;

        	@Override
        	public void onStart(long processId) {
                LOGGER.info("Strategy started: " + processId);
        	}

			@Override
			public void onStop(long processId) {
                LOGGER.info("Strategy stopped: " + processId);
                if (client.getStartedStrategies().size() == 0) {
                    System.exit(0);
                }
			}

			@Override
			public void onConnect() {
                LOGGER.info("Connected");
                lightReconnects = 3;
			}

			@Override
			public void onDisconnect() {
                Runnable runnable = new Runnable() {
                    @Override
                    public void run() {
                        if (lightReconnects > 0) {
                            client.reconnect();
                            --lightReconnects;
                        } else {
                            do {
                                try {
                                    Thread.sleep(60 * 1000);
                                } catch (InterruptedException e) {
                                }
                                try {
                                    if(client.isConnected()) {
                                        break;
                                    }
                                    client.connect(jnlpUrl, userName, password);

                                } catch (Exception e) {
                                    LOGGER.error(e.getMessage(), e);
                                }
                            } while(!client.isConnected());
                        }
                    }
                };
                new Thread(runnable).start();
			}
		});

        LOGGER.info("Connecting...");
        //connect to the server using jnlp, user name and password
        client.connect(jnlpUrl, userName, password);

        //wait for it to connect
        int i = 10; //wait max ten seconds
        while (i > 0 && !client.isConnected()) {
            Thread.sleep(1000);
            i--;
        }
        if (!client.isConnected()) {
            LOGGER.error("Failed to connect Dukascopy servers");
            System.exit(1);
        }

        //subscribe to the instruments
        LOGGER.info("Subscribing instruments...");
        client.setSubscribedInstruments(DataCenter.PairSet);
        int k = 10;
        while (!client.getSubscribedInstruments().containsAll(DataCenter.PairSet)) {
            try {
                LOGGER.info("Instruments not subscribed yet " + k);
                Thread.sleep(200);
            } catch (InterruptedException e) {
                LOGGER.error(e.getMessage());
            }
            k--;
        }
        
        //start the strategy
        LOGGER.info("Starting strategy");
        dataService = new DataService(); 
        client.startStrategy(dataService);
        //now it's running
        
        //UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        //UIManager.setLookAndFeel("com.bulenkov.darcula.DarculaLaf");
        //org.jb2011.lnf.beautyeye.BeautyEyeLNFHelper.launchBeautyEyeLNF();
        
        /*
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                FuckForexGui ex = new FuckForexGui();
                ex.setVisible(true);
            }
        });
        */
    }
}
