package singlejartest.chartcontrol;

import java.util.List;

import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;
import javafx.scene.shape.StrokeLineCap;
import javafx.scene.shape.StrokeLineJoin;
import javafx.scene.text.Font;
import singlejartest.data.Price;

public class Chart extends Canvas {
	
	private final GraphicsContext GFX = getGraphicsContext2D();
	
	private final int AxisX = 50;
	private final int AxisY = 40;
	private final int BarSpace = 5;
	
	public Chart() {
		super();
		
		GFX.setFont(new Font("Input", 12));
		GFX.setLineJoin(StrokeLineJoin.ROUND);
		GFX.setLineCap(StrokeLineCap.ROUND);
		GFX.setLineWidth(2);
	}
	
	public void setPrefSize(int width, int height) {
		setWidth(width); setHeight(height);
	}
	
	private PlotOptions mPlotOptions;
	
	public void setPlotOptions(PlotOptions plotOptions) {
		mPlotOptions = plotOptions;
	}
	
	public PlotOptions getPlotOptions() {
		return mPlotOptions;
	}

	private int mPeriod;
	
	public void setPeriod(int period) {
		mPeriod = period;
	}
	
	private boolean canSyncMouse = false;
	
	public void setSyncMouse(boolean value) {
		canSyncMouse = value;
	}
	
	private int mMouseX1;
	private int mMouseX2;
	private boolean canDrawMouse = false;
	
	public void renderMouse(int x1, int x2) {
		canDrawMouse = true;
		mMouseX1 = x1; mMouseX2 = x2;
		update(false);
	}
	
	private int mNumBars;
	
	public int getNumBars() {
		return (int) (getWidth() - AxisX) / BarSpace;
	}
	
	private int mChartWidth;
	private int mChartHeight;
	
	public void update(boolean hasNewBar) {
		mChartWidth = (int) (getWidth() - AxisX);
		mChartHeight = (int) (getHeight() - AxisY);
		mNumBars = mChartWidth / BarSpace;
		
		if(hasNewBar) {
			GFX.clearRect(0, 0, getWidth(), getHeight());
			GFX.setFill(Color.ANTIQUEWHITE);
			GFX.fillRect(0, 0, getWidth(), getHeight());
			
			renderTime();
			findMinMax();
		}
		else {
			GFX.clearRect(0, 0, mChartWidth, mChartHeight);
			GFX.setFill(Color.ANTIQUEWHITE);
			GFX.fillRect(0, 0, mChartWidth, mChartHeight);
		}
		
		GFX.setFill(Color.BLACK);
		GFX.fillText(mPlotOptions.Name, 10, 22);
		
		if(canSyncMouse && canDrawMouse) {
			GFX.setStroke(Color.GRAY);
			GFX.strokeLine(mMouseX1, 0, mMouseX1, mChartHeight);
			if(mMouseX2 - mMouseX1 > BarSpace) {
				GFX.strokeLine(mMouseX2, 0, mMouseX2, mChartHeight);
			}
		}
		
		renderLines();
	}
	
	private void findMinMax() {
		List<Price> series = mPlotOptions.Series;
		int count = series.size();
		int startIndex = count - mNumBars; if(startIndex < 0) { startIndex = 0; }
        double min = series.get(startIndex).Close, max = min;
        mPlotOptions.MinIndex = startIndex;
        mPlotOptions.MaxIndex = startIndex;
		for(int i = startIndex + 1; i < count; i++) {
			double p = series.get(i).Close;
			if(p < min) { min = p; mPlotOptions.MinIndex = i; }
			else if(p > max) { max = p; mPlotOptions.MaxIndex = i; }
		}
	}
	
	private void renderTime() {
		List<Price> series = mPlotOptions.Series;
		int count = series.size();
		int x = 0, y = mChartHeight + AxisY/2;
		GFX.setFill(Color.BLACK);
		for(int startIndex = count - mNumBars; startIndex < count; startIndex++) {
			long time = series.get(startIndex).Time;
			
			if(time % mPeriod == 0) {
				long hour = time % 86400000 / 3600000;
				long minute = time % 3600000 / 60000;
				if(minute == 0) {
					GFX.fillText(Long.toString(hour), x, y);
				}
				else {
					GFX.fillText(hour + ":" + minute, x, y);
				}
			}
			x += BarSpace;
		}
	}
	
	private void renderLines() {
		List<Price> series = mPlotOptions.Series;
		int count = series.size();
		int startIndex = count - mNumBars; if(startIndex < 0) { startIndex = 0; }
		
		double last = series.get(mPlotOptions.LastIndex).Close; 
		double min = series.get(mPlotOptions.MinIndex).Close;
        double max = series.get(mPlotOptions.MaxIndex).Close;
		
        if(last < min) { min = last; }
        else if(last > max) { max = last; }
        
		double scale = mChartHeight / (max - min);
		int x1 = 0, x2 = BarSpace, y1, y2;
		
		GFX.setStroke(mPlotOptions.Colour);
		for(startIndex++; startIndex < count; startIndex++) {
			y1 = (int)((max - series.get(startIndex - 1).Close) * scale);
			y2 = (int)((max - series.get(startIndex    ).Close) * scale);
			GFX.strokeLine(x1, y1, x2, y2);
			x1 += BarSpace; x2 += BarSpace;
		}
	}
}
